/*******************************************************************************
 * Copyright (c) 2015 Texas Instruments and others All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Gingrich, Paul - Initial API and implementation
 ******************************************************************************/
var gc = gc || {};
gc.databind = gc.databind || {};

(function() // closure for private static methods and data.
{
    var numPacketsReceived = 0;
    
    var SerialPortModel = function()
    {
        gc.databind.AbstractStreamingDataModel.call(this);
    };

    SerialPortModel.prototype = new gc.databind.AbstractStreamingDataModel('uart');

    var serialIO;
    
    SerialPortModel.prototype.sendValue = function(value, progress)
    {
        if (this._encoder)
        {
            var packet = this._encoder.encode(value);
            if (serialIO)
            {
                serialIO.serialSend(packet);
            }
            else
            {
                console.log('Discarded tx uart data: ' + packet);
            }
        }
    };

    var serialPortInstance = new SerialPortModel();
    var callbackForDataValidation;

    gc.databind.SerialPortModel =
    {
        serialPortRecievedHandler : 
        {
            text: function(message, buffer)
            {
                var decoder = serialPortInstance._decoder;
                if (decoder)
                {
                    decoder.decode(buffer, serialPortInstance);
                    var isConnected = decoder.isConnected();
                    serialPortInstance.setConnectedState(isConnected);
                    if (callbackForDataValidation && isConnected)
                    {
                        callbackForDataValidation();
                        callbackForDataValidation = undefined;
                    }
                }
                else
                {
                    console.error('SerialPortModel: data received before decoder initialized');
                }
            }
        },
        connect : function(dataValidationCallback, rxPacketDataFormat)
        {
            serialPortInstance._decoder = serialPortInstance.createCodec(rxPacketDataFormat); 
            callbackForDataValidation = dataValidationCallback;
        },
        disconnect : function()
        {
            callbackForDataValidation = undefined;
            serialPortInstance._encoder = undefined;
            serialPortInstance._decoder = undefined;
            serialPortInstance.setConnectedState(false);
        },
        setSerialIOInstance : function(serialIOInstance, txPacketDataFormat)
        {
            serialIO = serialIOInstance;
            serialPortInstance._encoder = serialPortInstance.createCodec(txPacketDataFormat); 
        }
    };

    // register serial model with bindingRegistry
    gc.databind.registry.registerModel(new gc.databind.StorageProviderBindFactoryWrapper(serialPortInstance), true);

}());
